from pymongo import MongoClient


class PTMongoDb(object):
    """
    reference: https://www.mongodb.com/languages/python
    tutorial: https://pymongo.readthedocs.io/en/stable/
    """
    def __init__(self, conn_string=None):
        self.conn_string = conn_string
        self.database = None
        self.table = None
        # self.conn_string = "mongodb://ptadmin:Slaj5WVdTnLPVXTb@dev-shard-00-00.azphn.mongodb.net:27017,dev-shard-00-01.azphn.mongodb.net:27017,dev-shard-00-02.azphn.mongodb.net:27017/pt-admin?authSource=admin&replicaSet=atlas-pn5t9g-shard-0&retryWrites=true&ssl=true&w=majority"

    # def create_table_if_not_existed(self, primary_key_schema, sort_key_schema=None):
    #     pass

    def put(self, primary_key, item, ttl=None):
        """
        put item into mongoDb, ttl is not supported yet
        :param primary_key: key name of <string>
        :param item: <dict>
        :param ttl: not support
        :return:
        """
        if "_id" in item:
            del item["_id"]
        resp = self.table.replace_one({"_id": primary_key}, item, upsert=True)
        return resp.upserted_id
        # existed = self.get(primary_key)
        # if existed:
        #     item.update(existed)
        #     resp = self.table.update_one({"_id": primary_key}, item)
        #     return resp
        # else:
        #     item.update({"_id": primary_key})
        #     return self.table.insert_one(item).inserted_id
        # self.table.insert_many([item])

    def get(self, primary_key):
        """
        get item from mongoDb
        :param primary_key: key name of <string>
        :return:
        """
        return self.table.find_one({"_id": primary_key})

    def delete(self, primary_key):
        """
        delete item from mongodb by primary_key
        :param primary_key: key name of <string>
        :return:
        """
        return self.table.delete_one({"_id": primary_key})

    def query(self, conditions):
        """
        query items by input conditions
        :param conditions: List<dict>
        :return:
        """
        resp = self.table.find(conditions)
        return resp

    def switch_table(self, db_name, table_name):
        # from pymongo import MongoClient
        # import pymongo

        # Provide the mongodb atlas url to connect python to mongodb using pymongo
        # CONNECTION_STRING = "mongodb://ptadmin:Slaj5WVdTnLPVXTb@dev-shard-00-00.azphn.mongodb.net:27017,dev-shard-00-01.azphn.mongodb.net:27017,dev-shard-00-02.azphn.mongodb.net:27017/pt-admin?authSource=admin&replicaSet=atlas-pn5t9g-shard-0&retryWrites=true&ssl=true&w=majority"

        # Create a connection using MongoClient. You can import MongoClient or use pymongo.MongoClient

        client = MongoClient(self.conn_string)

        # Create the database for our example (we will use the same database throughout the tutorial
        self.database = client[db_name]
        self.table = client[db_name][table_name]

    # collection = db["products"]


if __name__ == "__main__":
    conn_string = "mongodb://ptadmin:Slaj5WVdTnLPVXTb@dev-shard-00-00.azphn.mongodb.net:27017,dev-shard-00-01.azphn.mongodb.net:27017,dev-shard-00-02.azphn.mongodb.net:27017/pt-admin?authSource=admin&replicaSet=atlas-pn5t9g-shard-0&retryWrites=true&ssl=true&w=majority"
    mongodb = PTMongoDb(conn_string=conn_string)
    mongodb.switch_table(db_name="test", table_name="products2")
    item_1 = {
        "_id": "U1IT00003",
        "item_name": "Blender",
        "max_discount": "10%",
        "batch_number": "RR450020FRG111",
        "price": 341,
        "category": "kitchen appliance"
    }
    mongodb.put("U1IT00004", item_1)
    # collection = db["products"]
    item_details = mongodb.get("U1IT00003")

    items = mongodb.query({"item_name": "Blender"})
    for item in items:
        print(item)
    mongodb.delete("U1IT00003")
    # for item in item_details:
    #     # This does not give a very readable output
    #     print(item)
    # dbname =
    print("done")


from aws_xray_sdk.core import xray_recorder
from aws_xray_sdk.core import patch_all


class PTTrace(object):
    def __init__(self):
        pass

    @staticmethod
    def enable_all_trace():
        patch_all()

    @staticmethod
    def record(subsegment_name):
        return xray_recorder.in_subsegment(subsegment_name)

    """
    with xray_recorder.in_subsegment("Throwing exception") as subsegment:
        subsegment.put_metadata("very_important_inforamtion", {"message": "Hello I am metadata"}, "information")
        subsegment.put_annotation("very_important_annotation", "Very very important information")
        raise Exception("Information")
    """


import datetime

import boto3
from boto3.dynamodb.conditions import Key

dynamodb = boto3.resource('dynamodb')


class PTDynamoDb(object):
    def __init__(self, table_name):
        self.table_name = table_name

    def get(self, primary_key_item, sort_key_item=None):
        """
        get item from dynamoDb by primary key and sort key
        :param primary_key_item: {<KeyName>:<Value>}
        :param sort_key_item: {<KeyName>:<Value>}
        :return: dict
        """
        item = dict()
        item.update(primary_key_item)
        if sort_key_item:
            item.update(sort_key_item)
        response = dynamodb.Table(self.table_name).get_item(
            Key=item
        )
        obj = response.get("Item")
        if obj is None:
            return None
        enabled_ttl = obj.get("ExpirationTimestamp")
        if enabled_ttl:
            expired = int(datetime.datetime.utcnow().timestamp()) > item["ExpirationTimestamp"]
            if expired:
                return None
        return obj

    def put(self, primary_key_item, sort_key_item, item, ttl=None):
        """
        put item into dynamoDb, expired in ttl
        :param primary_key_item: {<KeyName>:<Value>}
        :param sort_key_item: {<KeyName>:<Value>
        :param item: dict
        :param ttl: integer in seconds
        :return:
        """
        utc_now = datetime.datetime.utcnow()
        item.update(primary_key_item)
        if sort_key_item:
            item.update(sort_key_item)
        item.update({"CreationTimestamp": int(utc_now.timestamp())})
        if ttl:
            item["ExpirationTimestamp"] = int((utc_now + datetime.timedelta(seconds=ttl)).timestamp())
        return dynamodb.Table(self.table_name).put_item(
            Item=item
        )

    def delete(self, primary_key_item, sort_key_item=None):
        """
        delete item where primary_key_name==primary_key_value and sort_key_name==sort_key_value
        :param primary_key_item:
        :param sort_key_item:
        :return:
        """
        key_item = dict()
        key_item.update(primary_key_item)
        if sort_key_item:
            key_item.update(sort_key_item)
        return dynamodb.Table(self.table_name).delete_item(
            Key=key_item
        )

    def get_or_default(self, primary_key_item, sort_key_item, default_item, ttl=None):
        """
        get item, put a default item if not existed
        :param primary_key_item:
        :param sort_key_item:
        :param default_item:
        :param ttl:
        :return:
        """
        obj = self.get(primary_key_item, sort_key_item)
        if obj is not None:
            return obj
        else:
            self.put(primary_key_item, sort_key_item, default_item, ttl)
            return self.get(primary_key_item, sort_key_item)

    def query_by_sort_key_range(self, primary_key_item, sort_key_item):
        """
        query_by_sort_key_range
        :param primary_key_item: {primary_key_name: primary_key_value}
        :param sort_key_item: {sort_key_name: {sort_key_range_start: "xxx", sort_key_range_end: "yyy}}
        :return:
        """
        primary_key_name = list(primary_key_item.items())[0][0]
        primary_key_value = list(primary_key_item.items())[0][1]
        sort_key_name = list(sort_key_item.items())[0][0]
        sort_key_range = list(sort_key_item.items())[0][1]
        sort_key_range_start = sort_key_range[0]
        sort_key_range_end = sort_key_range[1]
        response = dynamodb.Table(self.table_name).query(
            KeyConditionExpression=
            Key(primary_key_name).eq(primary_key_value) &
            Key(sort_key_name).between(sort_key_range_start, sort_key_range_end)
        )
        print("Done")
        pass

    def create_table_if_not_existed(self, primary_key_schema, sort_key_schema=None):
        """
        create_table_if_not_existed
        :param primary_key_schema: {primary_key_name: primary_key_type} -> primary_key_type: S | N | B
        :param sort_key_schema: {sort_key_name: sort_key_type}
        :return:
        """
        import time
        client = boto3.client('dynamodb')
        exists = True
        try:
            describeTableResp = client.describe_table(
                TableName=self.table_name
            )
            return self.table_name
        except Exception as ex:
            if ex.response["Error"]["Code"] == "ResourceNotFoundException":
                exists = False
            else:
                raise ex
        if not exists:
            primary_key_name = list(primary_key_schema.items())[0][0]
            primary_key_type = list(primary_key_schema.items())[0][1]
            # additional_keys = ["Timestamp", "SerialNo"]
            key_schema = [{
                "AttributeName": primary_key_name,
                "KeyType": "HASH"
            }]
            attribute_definitions = [{
                "AttributeName": primary_key_name,
                "AttributeType": primary_key_type
            }]
            if sort_key_schema:
                sort_key_name = list(sort_key_schema.items())[0][0]
                sort_key_type = list(sort_key_schema.items())[0][1]
                key_schema.extend([{"AttributeName": sort_key_name, "KeyType": "RANGE"}])
                attribute_definitions.extend([{"AttributeName": sort_key_name, "AttributeType": sort_key_type}])

            createTableResp = client.create_table(
                TableName=self.table_name,
                KeySchema=key_schema,
                AttributeDefinitions=attribute_definitions,
                ProvisionedThroughput={
                    "ReadCapacityUnits": 5,
                    "WriteCapacityUnits": 5
                }
            )
            status = "CREATING"
            while status != "ACTIVE":
                describeTableResp = client.describe_table(
                    TableName=self.table_name
                )
                status = describeTableResp["Table"]["TableStatus"]
                time.sleep(0.5)

            updateTTLResp = client.update_time_to_live(
                TableName=self.table_name,
                TimeToLiveSpecification={
                    'Enabled': True,
                    'AttributeName': 'ExpirationTimestamp'
                }
            )
            return createTableResp["TableDescription"]["TableName"]


if __name__ == "__main__":
    # db = PTDynamoDb(table_name="table_student")
    # db.create_table_if_not_existed(primary_key_schema={"No": "N"}, sort_key_schema={"Age": "N"})
    # db.put(primary_key_item={"No": 1}, sort_key_item={"Age": 10}, item={"Name": "Peter", "Score": 80})
    # obj = db.get(primary_key_item={"No": 1}, sort_key_item={"Age": 10})
    # db.delete(primary_key_item={"No": 1}, sort_key_item={"Age": 10})
    db = PTDynamoDb(table_name="amzProductDiagnosticJob")
    db.create_table_if_not_existed(primary_key_schema={"JobId": "S"}, sort_key_schema=None)
    item = db.get_or_default(primary_key_item={"JobId": "AAA"}, sort_key_item=None,
                             default_item={"IsDiagnosticJobCompleted": False, "AsinList": []})
    item["AsinList"].append("123")
    db.put(primary_key_item={"JobId": "AAA"}, sort_key_item=None, item=item)
    print(item)


import logging

root_logger = logging.getLogger()
if root_logger.handlers:
    for handler in root_logger.handlers:
        root_logger.removeHandler(handler)
logging.basicConfig(format='%(asctime)s [%(name)s] [%(levelname)s] %(message)s', level=logging.INFO)


class PTLogger(object):
    def __init__(self, name):
        self.name = name

    def get_logger(self):
        return logging.getLogger(self.name)



import boto3
import logging

from botocore.exceptions import ClientError

metric = boto3.client('cloudwatch')
logger = logging.getLogger(__name__)


class PTMetric:
    def __init__(self, namespace, dimensions=None):
        self.namespace = namespace
        self.dimension_list = [{"Name": k, "Value": v} for k, v in dimensions.items()] if dimensions else None

    def put_metric_data(self, name, value, unit):
        metric_data = {
            'MetricName': name,
            'Value': value,
            'Unit': unit
        }
        if self.dimension_list:
            metric_data.update({'Dimensions': self.dimension_list})
        metric.put_metric_data(
            Namespace=self.namespace,
            MetricData=[metric_data]
        )


if __name__ == "__main__":
    dimensions = {"Application": "Diagnosis", "Module": "Spider", "FunctionName": "func_name"}
    dimension_list = [{"Name": k, "Value": v} for k,v in dimensions.items()]
    print(dimension_list)


from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

with open('README.md', encoding='utf-8') as fp:
    long_description = fp.read()

# with open('requirements.txt', encoding='utf-8') as fp:
#     install_requires = fp.read()

setup(name='pt_lambda_util',
      version='0.1.8',
      description='common utility for aws lambda application',
      long_description=long_description,
      long_description_content_type='text/markdown',
      author='JimmyMo',
      author_email='jocund_mo@aliyun.com',
      install_requires=["boto3"],
      package_dir={'pt_lambda_util': 'src/pt_lambda_util'},
      packages=["pt_lambda_util"],
      # packages=["util"],
      # package_dir={"package1": ".\\util"},
      # package_data={"": ["*.*"]},
      license="Apache License 2.0"
      )
